package status_icon.handlers;

import java.util.Map;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.HandlerEvent;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.commands.IElementUpdater;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.menus.UIElement;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import status_icon.IStatusListener;
import status_icon.Status;
import status_icon.StatusService;

public class StatusHandler extends AbstractHandler implements IStatusListener, IElementUpdater {

    private final String commandId = "status_icon.commands.status";
    private static final ImageDescriptor icon_disabled = AbstractUIPlugin.imageDescriptorFromPlugin("status_icon",
            "icons/status_disabled.png");
    private static final ImageDescriptor icon_a = AbstractUIPlugin.imageDescriptorFromPlugin("status_icon",
            "icons/status_a.png");
    private static final ImageDescriptor icon_b = AbstractUIPlugin.imageDescriptorFromPlugin("status_icon",
            "icons/status_b.png");
    protected Status currentStatus = StatusService.getStatus();

    public StatusHandler() {
        StatusService.addListener(this);
    }

    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchWindow window = HandlerUtil.getActiveWorkbenchWindowChecked(event);
        MessageDialog.openInformation(window.getShell(), "Status", "Current status: " + currentStatus);
        return null;
    }

    @Override
    public void updateStatus(Status newStatus) {
        this.currentStatus = newStatus;
        // "update tooltip" works when handler is enabled (see eclipse bug 275126)
        // update "enabled" status
        fireHandlerChanged(new HandlerEvent(this, true, false));
        // update tooltip and icons
        ICommandService cs = (ICommandService) PlatformUI.getWorkbench().getService(ICommandService.class);
        cs.refreshElements(commandId, null);
    }

    @Override
    public boolean isEnabled() {
        return currentStatus == Status.A || currentStatus == Status.B;
    }

    public void updateElement(UIElement element, @SuppressWarnings("rawtypes") Map parameters) {
        // boolean enabled = currentStatus != null;
        String tooltip = "Status: " + currentStatus;
        ImageDescriptor icon;
        if (currentStatus == Status.A) {
            icon = icon_a;
        } else if (currentStatus == Status.B) {
            icon = icon_b;
        } else {
            icon = icon_disabled;
        }

        element.setTooltip(tooltip);
        element.setIcon(icon);
        element.setDisabledIcon(icon);
    }

    public void dispose() {
        StatusService.removeListener(this);
        super.dispose();
    }

}
